library(moments)
library(gsg)
library(mgcv)
library(ggplot2)

NbSimul <- c(1:100) # number of replicats 
valAsym <- c(0.001, 0.5, 1, 2, 3, 4, 5) # skewness of genetic distribution  
valNb <- c(200, 1000) # sample size 
valSd <- c(0, 1, 2, 5) # environmental variance 

data <- NULL

for (i in valAsym){
  for (j in valNb){
    for (l in valSd){
      for (p in 1:length(NbSimul)){
        
        ### Breeding value distribution
        vg = 1 # genetic variance 
        k = ((2/i)^2) # shape parameter of Gamma distribution
        theta = (sqrt(vg)/sqrt(k)) # scale parameter of Gamma distribution
        3g = (i*(sqrt(vg)^3)) # third order moment of breeding values
        
        g <- rgamma(j, shape = k, scale = theta) # breeding values 
        
        ###  Position of mean breeding values (= phenotypic mean) mean from the adaptive optimum (DOpt)
        DOpt = (-((vg)^2)/(sqrt(k))) # distance between mean phenotype an optimum at equilibrium
        opt = 0 # adaptive optimum
        gbar = DOpt + opt # mean breeding values
        g <- g - mean(g) + gbar 
        
        ### Phenotypic distribution  
        e <- rnorm(j, mean = 0, sd = sqrt(l)) # environmental effects
        z <- g + e # phenotypes
        
        ### Realized parameters
        vg_real = var(g) # realized variance of breeding values
        ve_real = var(e) # realized variance of environmental effetcs
        vz_real = var(z) # realized variance of phenotype 
        skew_g_real = skewness(g) # realized skewness of breeding values
        skew_e_real = skewness(e) # realized third central moment of environmental effects
        skew_z_real = skewness(z) # realized phenotypic third central moment
        3g_real = (skew_g_real*(sqrt(vg_real)^3)) # realized third central moment of breeding values
        3e_real = (skew_e_real*(sqrt(ve_real)^3)) # realized third central moment of environmental effects
        3z_real = (skew_z_real*(sqrt(vz_real)^3)) # realized third central moment of phenotype
        
        ### Fitness 
        wmax = 10 # maximal fitness 
        om = 5 # peak width of the fitness function 
        opt = 0 # adaptive optimum
        
        # Stabilizing fitness function
        funcfit <- function(x) {   
          (wmax*(1-(((x-opt)^2)/(2*(om^2)))))
        }
        
        wBrute <- funcfit(z) # fitness of individual for phenotype z
        wBruteSS <- ifelse(wBrute<0, 0.00001, wBrute) # to remove negative fitness
        
        # Robertson-Price identity
        wBruteRel <- wBrute/mean(wBrute) # relative fitness
        Rob <- cov(g,wBruteRel) # Robertson-Price identity
        
        g_round <- round(g, 1)
        blah <- as.data.frame(cbind(wBruteRel, g_round))
        meanWGtranche <- aggregate(blah, by = list(blah$g_round),mean) 
        out <- g_round
        for(f in meanWGtranche$g_round){
          out[out==f] <- meanWGtranche$wBruteRel[which(meanWGtranche$g_round==f)]
        }
        RobGValUnique <- cov(g,out) # Robertson-Price identity using mean fitness of a breeding values
        RobGTranche <- cov(g_round, out)
        
        # Realized fitness
        wReal <- rpois(j,wBruteSS) # number of offspring
        moywReal <- mean(wReal, na.rm=T) # mean realized fitness of the samples
        moywRealSS <- ifelse(moywReal==0, 0.001, moywReal)
        wRel <- wReal/moywRealSS # relative realized fitness
        
        ### Selection estimations 
        
        # Ordinary Least Square regression (OLS, Lande & Arnold Method)
        
        # Directional selection
        regBetaEst =  lm(wRel~z)
        resumBetaEst = summary(regBetaEst)
        betaEst = resumBetaEst$coefficients[2,1] # directional selection gradient
        betaSt <- betaEst*sqrt(vz_real) # standardized directional selection gradient
        regBetaPval = glm(wReal~z, family = poisson)
        resumBetaPval = summary(regBetaPval)
        betaPval = resumBetaPval$coefficients[2,4] # p-value of the directional selection gradient estimated
        
        # Quadratic selection
        regGammaEst = glm(wRel~z+I(z^2), family = gaussian)
        resumGammaEst = summary(regGammaEst)
        gammaEst = 2*resumGammaEst$coefficients[3,1] # quadratic selection gradient
        gammaSt <- gammaEst*(vz_real) # standardized quadratic selection gradient
        regGammaPval = glm(wReal~z+I(z^2), family = poisson)
        resumGammaPval = summary(regGammaPval)
        gammaPval = resumGammaPval$coefficients[3,4]
        GammaEstPOIS = 2*resumGammaPval$coefficients[3,1] # p-value of the quadratic selection gradient estimated

        # Generalize Additive Model (ADIRF, Morrissey & Sakrejda Method)
        
        regGAM <- gam(wRel~s(z)) # Generalize Additif Model
        grad <- gam.gradients(regGAM, phenotype="z", se.method="permute", n.boot = 100)
        grad <- as.data.frame(grad$ests)
        betaEstGAM <- grad[1,1] # directional selection gradient
        betaPvalGAM <- grad[1,3] # p-value of the directional selection gradient estimated
        gammaEstGAM <- grad[2,1] # quadratic selection gradient 
        gammaPvalGAM <- grad[2,3] # p-value of the quadratic selection gradient estimated
        
        ### Theoretical standardized selection gradients
        
        # Theoretical selection gradient
        BetaAtTh = (-(2*(vg^2)*sqrt(k))/((vg^4)+(vg*k)+(k*(l-(2*om^2)))))*sqrt((vg+l)) # Expected directional selection gradient
        GammaAtTh = ((2/(vg + l + (gbar - opt)^2 - 2*om^2))*(vg+l)) # Expected quadratic selection gradient
        
        # Theoretical selection gradient from the OLS method
        DOpt = -3g_real/(2*vg_real)
        S = 1/(om^2-(vg_real+ve_real+DOpt^2)/2)
        BetaAtTh_OLS_ = -S*(DOpt+(3z_real)/(2*(vg_real+ve_real)))*sqrt(vg_real+ve_real) # Expected directional selection gradient with OLS method
        
        ### Theoretical response to selection
        S = 1/(om^2-(vz_real+DOpt^2)/2)
        RespG = -S*(DOpt*vg_real+(3g_real/2)) # Expected response to selection from Burger
        RespClassic <- vg_real*betaSt # Expected response to selection from the breeder's equation (OLS method)
        
        vec <- c(i, l, j, p, skew_g_real, skew_z_real, skew_e_real, vg_real, vz_real, ve_real, betaEst, betaPval, betaSt, BetaAtTh, BetaAtTh_OLS, Rob, RespG, RespClassic, gammaEst, gammaPval, BetaAtTh, GammaAtTh, betaEstGAM, betaPvalGAM, gammaEstGAM, gammaPvalGAM, RobGValUnique, RobGTranche)
        
        data <- rbind(data, vec)
      }   
    }
  }
}

data <- as.data.frame(data)

names(data)[1] <- "skew_g"; names(data)[2] <- "ve"; names(data)[3] <- "nbInd"; names(data)[4] <- "repl"; names(data)[5] <- "skew_g_real"; names(data)[6] <- "skew_z_real"; names(data)[7] <- "skew_e_real"; names(data)[8] <- "vg_real"; names(data)[9] <- "vz_real"; names(data)[10] <- "ve_real"; names(data)[11] <- "beta"; names(data)[12] <- "betaPval"; names(data)[13] <- "betaSt"; names(data)[14] <- "BetaAtTh"; names(data)[15] <- "BetaAtTh_OLS"; names(data)[16] <- "Rob"; names(data)[17] <- "RespG"; names(data)[18] <- "RespClassic"
names(data)[19] <- "gammaEst"; names(data)[20] <- "gammaPval"; names(data)[21] <- "BetaAtTh"; names(data)[22] <- "GammaAtTh"; names(data)[23] <- "betaEstGAM"; names(data)[24] <- "betaPvalGAM"; names(data)[25] <- "gammaEstGAM"; names(data)[26] <- "gammaPvalGAM"; names(data)[27] <- "RobGValUnique"; names(data)[28] <- "RobGTranche"

write.table(data, file = "DataSimu_SkewG.txt", col.names = T, row.names = F ,append = T, sep="\t")
